#ifndef CHECK_UC_CERT_COLLECTOR_HPP
#define CHECK_UC_CERT_COLLECtOR_HPP

#include <string>
#include <vector>
#include "AKCertHolder.hpp" 
#include "Policy.hpp" 
#include "AkentiMessage.hpp" 
#include "LogInterface.hpp" 
#include "AKRuntime.hpp"
#include "CertInfoCollector.hpp"


using std::string;
using std::vector;
using namespace Akenti;

namespace Akenti {

class AKRuntime;

/**
  * A <i>CheckUCCertCollector</i> collects <i>UseCondition<i> certificates
  * and verifies them.
  *
  * The CheckUCCertCollector does not verify the policy certificates.
  * It assumes that this has been done by the PolicyCollector.
  *
  * @see Policy 
  * @see AKRuntime 
  *
  * @version 1.1 04/05/19
  * @author Sushila Subramanian
  * @author Abdelilah Essiari
  * @author Srilekha Mudumbai 
  */

class CheckUCCertCollector : public LogInterface {
 
public:

  /**
    * Contructs this object.
    *
    * @see AKRuntime
    */
  CheckUCCertCollector(AKRuntime* akRuntime);

  /**
    * Destroys this object.
    */
  ~CheckUCCertCollector();
  
  /**
    * Collects and verifies Use Condition certificates for each policy.
    *
    * @see collectUseCondCertificates(Policy&)
    */
  bool collectUseCondCertificates();

  /** 
    * Returns a status message.
    *
    * @see AkentiMessage
    */
  const AkentiMessage& getAkentiMessage() const;

private:

  /**
    * Populates the policy with verified Use Condition certficates.
    * Adds all use condition certificates encountered along the 
    * way to a CertInfo object that in turn is added to the vector
    * the CertInfoCollector keeps. Also adds messages to the 
    * <i>CertInfo</i> object to which the Use Condition Certificate
    * was added to tell if there was an error in processing the 
    * Certificate or if it was okay. 
    * 
    * @see getUCCertHolders(Policy&)
    */
  bool collectUseCondCertificates(Policy& policy);

  /**
    * Returns the UseCondition certficates.
    *
    * Algorithm to collect UseCondition certificates.
    *
    * <pre>
    *
    *         We collect the UseCondition certificates from the directories
    *         specified in the policy certificate. The collection process obeys
    *         the following rules. 
    *
    *         For each group of <i>UseConditionIssuer</i>'s we must find 
    *         a valid UseCondition certificate. That is a certificate that
    *         has not expired, is issued by one of the issuers for that
    *         group, and is found to be trusted. 
    *      
    *         Furthermore if we find one valid UseCondition in a directory,
    *         we assume that that directory contains all the relevant Use
    *         conditions for the resource. i.e. Each Use Condition directory
    *         listed must contain a complete set of all the Use Conditions.
    *
    *         If an error is encountered, the certificate is added to 
    *         the <i>CertInfo</i> object anyway, along with an AkentiMessage
    *         that stated what the error was.
    * </pre>
    *
    */
  vector<AkentiCertificate> getUCCertHolders(Policy& policy, vector <string>& locationVec);

  AKRuntime* akRuntime;

  AkentiMessage message;

};

} // end of name space

#endif /* CHECK_UC_CERT_COLLECTOR_HPP */
