#ifndef EXPRESSION_HPP
#define EXPRESSION_HPP

#include "ref_ptr.hpp"
#include "TreeNode.hpp"

#include <string>

using std::string;
using namespace Akenti;

namespace Akenti {

class ExpressionEvaluator;
class ExpressionVisitor;

/**
  *  This class is used to parse a string boolean expression into an internal
  *  tree represenation and to evaluate or reduce the expression for 
  *  a given user.
  * <pre>
  * Grammar:
  *
  * EXPR      ->  AEXPR   { OR AEXPR }
  * AEXPR     ->  NEXPER  { AND NEXPR }
  * NEXPER    ->  { NOT NEXPR } | PEXPER
  * PEXPER    ->  LEXPER AOP REXPER | ( EXPER )
  * LEXPER    ->  one non-empty string
  * REXPER    ->  one or more non-empty strings
  * AOP       ->  = | != | < | <= | > | >=
  * NOT       ->  ! 
  * AND       ->  && | & 
  * OR        ->  || | | 
  *
  * Precedence:
  *
  * Following is the list of operators we support. They 
  * are listed from highest to lowest precedence.
  *
  * { =, != , <, <= , >, >= }, !, { &&, &}, { ||, | }.
  * 
  * Parenthesis are of course optional. And as usual they can be used
  * for groupings.
  *  
  * Blank spaces have no meaning unless they are used to delimit the 
  * right hand side of an equality/or inequality operator. This is really
  * where we differ from the C expression parser.
  *
  * The following two partial expressions have the same meaning:
  *   <i> CN   =     Denis   The    Menace     </i>
  *   <i>CN=Denis The Menace</i>
  * </pre> 
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/
class Expression {

public:

  /**
    * Constructor.
    */
  Expression();

  /**
    * Constructs this <i>Expression</i> given <i>expression</i>.
    *
    * @param expression string that holds the boolean expression 
    * @exception ParserException if the expression does not 
    *            conform to the grammar.
    */
  Expression(const string& expression);

  /**
    * Destroys this object.
    */
  ~Expression();

  /**
    */
  bool isNull();
  
  /**
    * Returns a string representation of this <i>Expression</i>.
    *
    */
  string toString() const;

  /**
    * For logging purpose.
    */
  string toLogString() const;

  /**
    * Use this method when it is possible for the evaluator
    * to evaluate all non-boolean expressions.
    *
    */
  bool evaluate(ExpressionEvaluator& evaluator) const;

  /**
    * Use this method when it may not be possible for the evaluator
    * to evaluate all non-boolean expressions.
    * 
    * The resulting expression is placed in the out parameter <i>expr</i>
    * @return 1 or 0 if it was possible to evaluate the whole tree.
    *         -1 otherwise and the resulting expression is placed in expr.
    */
  int reduce(ExpressionEvaluator& evaluator, Expression& expr) const;

  /**
    * Use this method to visit all the non-boolean expressions.
    * 
    */
  void visit(ExpressionVisitor& visitor) const; 

private:

  /**
    * @exception Error
    */
  string toString(const TreeNode* n) const;

  /**
    * @exception Error
    */
  bool evaluate(const TreeNode* n, ExpressionEvaluator& evaluator) const;

  /**
    * @exception Error
    */
  int reduce(const TreeNode* n, ExpressionEvaluator& evaluator, TreeNode** out) const;

  /**
    * @exception Error 
    */
  void visit(const TreeNode* n, ExpressionVisitor& visitor) const;

  ref_ptr<TreeNode> root;

};

} // end of name space

#endif
