#ifndef AK_MESSAGE_DIGEST_HPP
#define AK_MESSAGE_DIGEST_HPP
#include "ref_ptr.hpp"
#include "MessageDigestImpl.hpp"
#include "Provider.hpp"

namespace Akenti {

/**
  * A <i>MessageDigest<i> object is a <i>reference-count</i> pointer 
  * that wraps a <i>MessageDigestImpl</i> object. It provides applications, 
  * albeit indirectly,  with the functionality of a message digest algorithm
  * such as MD5 or SHA1. Message digests are secure one-way hash functions 
  * that take arbitrary-sized data and output a fixed-length hash value.
  *
  * <PRE>
  * <b>Usage:</b> 
  *      try {
  *        MessageDigest md = MessageDigest::getInstance("MD5");
  *
  *        md->update("Message");
  *        md->update("Digest");
  *
  *        string hash = md->digest();
  *      } catch(SecurityException e) { ... }
  * </PRE>
  *
  * @see ref_ptr
  * @see MessageDigestImpl
  * @see Provider#getProvider()
  * @see ProviderImpl#getMessageDigest(const string&)
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

class MessageDigest : public ref_ptr<MessageDigestImpl> {

public:

  /**
    * Constructs a <i>MessageDigest</i> given a <i>MessageDigestImpl</i>
    * object. Typically the <i>getInstance(const string&)</i> method is
    * used to create an instance of this class.  
    *
    * @param impl  wrapped <i>MessageDigestImpl</i> object
    * @see ref_ptr
    * @see MessageDigestImpl
    * @see getInstance(const string&)
    */
  MessageDigest(MessageDigestImpl* impl = NULL) 
    : ref_ptr<MessageDigestImpl> (impl) {
  }

  /**
    * Destroys this object.
    *
    * @see ref_ptr
    */
  ~MessageDigest() {
  }

  /** 
    * Returns a <i>MessageDigest</i> object given the name of an
    * algorithm specified by <i>alg</i>. 
    *
    * @param alg string name of the algorithm
    * @exception SecurityException if algorithm is not supported 
    * @see Provider#getProvider()
    * @see ProviderImpl#getMessageDigest(const string&)
    */
  static MessageDigest getInstance(const string& alg) {
    Provider& po = Provider::getProvider();

    return po->getMessageDigest(alg);
  }

  /** 
    * A convenient method that returns a hash given 
    * the name of an algorithm specified by <i>alg</i> and the string 
    * to be hashed specified by <i>data</i>.
    *
    * @param alg string name of the algorithm 
    * @param data string to be hasheD
    * @exception Error if provider has not been installed 
    * @exception SecurityException if algorithm is not supported 
    * @see getInstance(const string& alg)
    */
  static string digest(const string &alg, const string &data) {
    MessageDigest md = getInstance(alg);

    md->update(data);
    return md->digest();
  }

  /**
    * Returns a Null Digest engine.
    */
  static MessageDigest getNullDigest() {
    return new MessageDigestImpl("NULL");
  }

};

} // end of namespace Akenti

#endif
