#ifndef AK_PROVIDER_HPP
#define AK_PROVIDER_HPP 
#include "ref_ptr.hpp"
#include "ProviderImpl.hpp"
#include "SecurityException.hpp"
#include "Error.hpp"

namespace Akenti {

/**
  * A <i>Provider</i> object is a <i>reference-count</i> pointer  
  * that wraps a <i>ProviderImpl</i> object. It provides applications,
  * albeit indirectly, with objects that implement security algorithms 
  * such as <i>MessageDigest</i>, <i>Base64encoding</i>
  * and <i>Signature</i>. At start-up a Provider is instantiated from a
  * selected ProviderImpl, e.g. OpenSSLProviderImpl, and installed as the
  * single static provider. Any attempt to reinstall a provider
  * will fail. As a result, the code can call static methods to perform
  * various security algorithms, but there is still some flexibilty as to
  * what implementations to use for these algorithms.
  *
  * @see ref_ptr
  * @see ProviderImpl 
  * @see MessageDigest 
  * @see Signature 
  * @see KeyFactory 
  * @see CertificateFactory 
  * @see SecurityException 
  * @see Error 
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class Provider : public ref_ptr<ProviderImpl> {

public:

  /**
    * Constructs a <i>Provider</i> given a <i>ProviderImpl</i> object.
    * Typically, an application installs a provider at start-up using the 
    * the <i>setProvider(Provider&)</i> method. Clients need never hold
    * a reference to this installed provider. The static <i>getProvider()</i> 
    * method can be used to get a reference to the installed provider.
    *
    * @param impl wrapped <i>ProviderImpl</i> object
    * @see ref_ptr
    * @see ProviderImpl 
    * @see setProvider(Provider&) 
    * @see getProvider() 
    */
  Provider(ProviderImpl* impl = NULL) : ref_ptr<ProviderImpl> (impl) {
  }

  /**
    * Destroys this object.
    *
    * @see ref_ptr
    */
  ~Provider() {
  }

  /**
    * Installs a <i>Provider</i>. This can only be called once in the lifetime 
    * of a program. Usually, this call should be made at start-up.
    *
    * @param prov the provider to be installed. 
    * @exception SecurityException if a provider is already installed
    */
  static void setProvider(const Provider& prov) {
    if (!prov) {
      throw Error("Provider:getProvider:invalid");
    } else if (Provider::provider) {
      throw SecurityException("Provider:setProvider:cannot reinstall provider");
    }

    Provider::provider = prov;
  }

  /**
    * Returns the installed provider.This call will fail 
    * if a provider has not been installed.
    *
    * @return the pre-installed provider
    * @exception Error if no provider has been installed 
    * @see setProvider(Provider&)  
    */
  static Provider& getProvider() {
    if (!provider) {
      throw Error("Provider:getProvider:provider not installed");
    }

    return provider;
  }
  
  static bool isProviderInstalled() {
    return provider != (Provider *) NULL;
  }

private:

  static Provider provider;

};

} // end of namespace Akenti

#endif
