#ifndef AK_SIGNATURE_IMPL_HPP
#define AK_SIGNATURE_IMPL_HPP
#include "PublicKey.hpp"
#include "PrivateKey.hpp"
#include "Error.hpp"

namespace Akenti {

/**
  * This abstract class provides applications with the interface
  * of a digital signature algorithm such as RSA-MD5 or DSA-SHA1. It also
  * provides some of the functionality. The rest of the functionality
  * is to be provided by the subclass implementors. Typically the clients
  * will not deal with an instance of this class directly. Instead they
  * interact with the <i>Signature</i> reference-count pointer.
  * <p>
  * A <i>SignatureImpl</i> object is initialized by calling either the <i>
  * intSign</i> if we are signing or the <i>initVerify</i> method if we are
  * verifying. The data is processed through it using the <i>update</i> methods.
  * Once all the data has been updated, one of the <i>sign</i> methods will be 
  * should be called to compute the signature, or the </i>verify</i> method  
  * should be called to verify a signature.
  *
  * @see Signature
  * @see Error
  * @see Provider#getProvider()
  * @see ProviderImpl#getSignature(const string&)
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/


class SignatureImpl {

public:

  /** 
    * Deletes this object.
    */
  virtual ~SignatureImpl() {
  }
  
  /**
    * Returns a string that identifies the algorithm, independent of
    * implementation details. The name should be a standard
    * name such as "RSA-MD5".
    *
    * @return name of the algorithm
    */
  virtual string getAlgorithm() const = 0;

  /**
    * Initializes this object for signing. If this method is called
    * again with a different argument, it negates the effect
    * of this call.
    *
    * @param privateKey the private key of the identity whose signature
    *        is going to be generated
    * @see resetEngine()
    */
  void initSign(const PrivateKey& privKey) {
    state = SIGN;
    this->privateKey = privKey;
    resetEngine();
  }

  /**
    * Initializes this object for verification. If this method is called
    * again with a different argument, it negates the effect of this call.
    *
    * @param publicKey the public key of the identity whose signature is
    *        going to be verified
    * @see resetEngine()
    */
  void initVerify(const PublicKey& pubKey) {
    state = VERIFY;
    this->publicKey = pubKey;
    resetEngine();
  }

  /**
    * Updates the data to be signed or verified.
    *
    * @param data the string of bytes to use for the update.
    * @see doUpdate(const string&, int, int)    
    */
  void update(const string& data) {
    doUpdate(data, 0, data.length());
  }

  /**
    * Updates the data to be signed or verified.
    *
    * @param data the string of bytes to use for the update
    * @param offset start index
    * @param len the number of bytes to use starting at <i>offset</i>
    * @exception Error if input buffer is too short
    * @see doUpdate(const string&, int, int)    
    */
  void update(const string& data, int offset, int len) {
    if (data.length() - offset < len) {
      throw Error("SignatureImpl:update:input buffer too short");
    }

    doUpdate(data, offset, len);
  }

  /**
    * Returns the digital signature of whatever data has been 
    * accumlated from the update calls.
    *
    * @exception Error if <i>initSign</i> has not been called 
    *                  or if the private key is uninitialized   
    * @exception SecurityException if signature fails
    * @see doSign()
    */
  string sign() {
    if (state != SIGN) {
      throw Error("SignatureImpl:sign:illegal state");
    }

    return doSign();
  }

  /**
    * Returns the digital signature of whatever data has been 
    * accumlated from the update calls plus the input data.
    *
    * @param data string of bytes to use for update
    * @exception Error if <i>initSign</i> has not been called 
    *                  or if the private key is uninitialized   
    * @exception SecurityException if signature fails
    * @see doUpdate(const string&, int, int)
    * @see sign()
    */
  string sign(const string& data) {
    doUpdate(data, 0, data.length());
    return sign();
  }

  /**
    * Returns the digital signature of whatever data has been 
    * accumlated from the update calls plus the input data.
    *
    * @param string & data string of bytes to use for update
    * @param int offset into string of data to use
    * @param int len number of bytes of data to use in update call
    * @exception Error if <i>initSign</i> has not been called 
    *                  or if the private key is uninitialized   
    * @exception SecurityException if signature fails
    */
  string sign(const string& data, int offset, int len) {
    update(data, offset, len);
    return sign();
  }

  /**
    * Returns true if signature verifies. False otherwise.
    *
    * @param signature digital signature to be verified
    * @exception Error if <i>initVerify</i> has not been called
    *                  or if the public key is uninitialized   
    * @see doVerify(const string&)
    */
  bool verify(const string& signature) {
    if (state != VERIFY) {
      throw Error("SignatureImpl:verify:Illegal state");
    }

    return doVerify(signature);
  }

protected:

  /**
    * Constructs this object.
    */
  SignatureImpl() {
  }

  /**
    * Resets the digest engine.
    */
  virtual void resetEngine() = 0;

  /**
    * Updates the data to be signed or verified.
    *
    * @param data the string of bytes to use for the update
    * @param offset start index
    * @param len the number of bytes to use starting at <i>offset</i>
    */
  virtual void doUpdate(const string& data, int offset, int len) = 0;

  /**
    * Returns the digital signature.
    *
    * @exception Error if the private key is uninitialized   
    * @exception SecurityException if signature fails
    */
  virtual string doSign() = 0;

  /**
    * Returns true if signature verifies. False otherwise.
    *
    * @param signature digital signature to be verified
    * @exception Error if the public key is uninitialized   
    */
  virtual bool doVerify(const string& signature) = 0;


  /**
    * Returns a string representation for SignatureImpl which is used for
    * debugging purposes.
    *
    * @return debugging info
    */
  virtual string paramString() const = 0; 

private:

  enum State { UNINITIALIZED, SIGN, VERIFY } state;

protected:

  PublicKey publicKey;       // used for verifying

  PrivateKey privateKey;     // used for signing

};

} // end of namespace Akenti

#endif
