#ifndef USE_COND_COLLECTOR_HPP
#define USE_COND_COLLECTOR_HPP

#include <string>
#include <vector>
#include "AKCertHolder.hpp" 
#include "Policy.hpp" 
#include "AkentiMessage.hpp" 
#include "LogInterface.hpp" 

using std::string;
using std::vector;
using namespace Akenti;

namespace Akenti {

class AKRuntime;

/**
  * A <i>UseCondCollector</i> collects <i>UseCondition<i> certificates
  * and verifies them.
  *
  * The UseCondition Collector does not verify the policy certificates.
  * It assumes that this has been done by the PolicyCollector.
  *
  * @see Policy 
  * @see AKRuntime 
  *
  * @version 1.1 00/05/01
  * @author Abdelilah Essiari
  * @author Srilekha Mudumbai 
  */

class UseCondCollector : public LogInterface {
 
public:

  /**
    * Contructs this object.
    *
    * @see Policy
    */
  UseCondCollector(AKRuntime* akRuntime);

  /**
    * Destroys this object.
    */
  ~UseCondCollector();
  
  /**
    * Collects and verifies uc certificates for each policy.
    *
    * @see collectUseCondCertificates(Policy&)
    */
  bool collectUseCondCertificates();

  /** 
    * Returns a status message.
    */
  const AkentiMessage& getAkentiMessage() const;

private:

  /**
    * Populates the policy with verified uc certficates.
    * The locally scoped uc certs will only be added 
    * if the policy pertains to a leaf resource.
    * 
    * @see getUCCertHolders(Policy&)
    */
  bool collectUseCondCertificates(Policy& policy);

  /**
    * Returns the UseCondition certficates.
    *
    * Algorithm to collect UseCondition certificates.
    *
    * <pre>
    *
    *    1) First we check the cache for UseCondition certificates.
    *
    *         The fact that these certs are found in the cache implies 
    *         that the condition that one uc must be found and must be valid 
    *         for each group is satisied.
    *
    *    2) Otherwise we search the directories. 
    *
    *         We collect the UseCondition certificates from the directories
    *         specified in the policy certificate. The collection process obeys
    *         the following rules. 
    *
    *         For each group of <i>UseConditionIssuer</i>'s we must find 
    *         a valid UseCondition certificate. That is a certificate that
    *         has not expired, is issued by one of the issuers for that
    *         group, and is found to be trusted. 
    *      
    *         Furthermore if we find one valid UseCondition in a directory,
    *         we assume that that directory contains all the relevant Use
    *         conditions for the resource. i.e. Each Use Condition directory
    *         listed must contain a complete set of all the Use Conditions.
    * </pre>
    *
    */
  vector<AKCertHolder> getUCCertHolders(Policy& policy);

  AKRuntime* akRuntime;

  AkentiMessage message;

};

} // end of name space

#endif
