#ifndef AK_REF_PTR_HPP
#define AK_REF_PTR_HPP

#include "Error.hpp"

/**
  * This class implements a simple reference counting scheme
  * that simplifies memory management. The ref_ptr object contains
  * a reference to another object, (the wraped pointer) and a count of 
  * how many places this ref_ptr is stored. One should avoid using 
  * the wrapped pointer directly. More important, one should 
  * never, and I mean never, hold a live reference to wrapped 
  * pointer since that will invalidate the reference count in the ref pointer.
  * We do allow null pointers to be wrapped. But the 
  * accesor methods throw exceptions when this pointer is null.
  * ref_ptr is defined using class templates, so that you can create a ref_ptr
  * to any class.
  *
  * Most of the Akenti classes that represent data objects are implented as
  * a set of classes. There is a class that should be used for external
  * manipulation of the object and is implemented as a ref_ptr. There is a
  * class that holds the data members implements the methods.By convention
  * the implementation class names end in Impl.
  *
  * <pre>
  * Usage:
  * class AImpl{
  *     f();
  *     g();
  * }
  *
  *  ref_ptr<AIimpl> aptr(new AImpl);
  *  aptr->f();
  *  aptr->g();
  * </pre>
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

namespace Akenti {

template<class T>
class ref_ptr {

public:

  /** 
    * Constructs a reference-count pointer. The ref_cnt starts at 1.
    *
    * @param pointee pointer to be wrapped
    **/
  ref_ptr(T *pointee = NULL);

  /**
    * Constructs a shallow copy of this object. Increments the ref_cnt of
    *   rhs by 1.
    * 
    * @param rhs the object to be copied
    **/
  ref_ptr(const ref_ptr& rhs);

  /**
    * Assignment operator. The ref_cnt of the target pointer is decremented.
    * the ref_cnt of the rhs is incremented.
    ***/
  ref_ptr<T>& operator=(const ref_ptr& rhs);

  /**
    * Destructor. Decrements the ref_cnt and when the ref_cnt is 0, deletes
    *  the object that the wrapper pointer references.
    **/
  ~ref_ptr();

  /**
    * Accesses the wrapped pointer. This is preferred over the 
    * <i>get</i> method. So the objects that are wrapped by ref_ptr
    * such as certificates should be referenced using the ref_ptr
    * e.g. Certificate->getType() rather CertificateImpl.getType()
    * 
    * @exception Error if the wrapped object is null
    * @see get()
    **/
  T* operator->() const;

  /**
    * Overloads the void * pointer for conditional testsing
    *
    * @return pointee
    */
  operator void*() const;

  /**
    * Returns the wrapped pointer. 
    * 
    * @return pointee
    * @exception Error if the wrapped object is null
    **/
  T* get() const;

private:

  void increment() {
    proxy->increment();
  }

  void decrement() {
    proxy->decrement();
    
    if (!proxy->getReferences()) {
      delete(proxy);
      proxy = NULL;
    }
  }

  struct Proxy {
    
    Proxy(T *pointee) : pointee(pointee), references(1) {
    }

    ~Proxy() {
      delete(pointee);
    }
  
    T* getPointee() {
      return pointee;
    }

    int getReferences() {
      return references;
    }

    void increment() {
      ++references;
    }

    void decrement() {
      --references;
    }

  private:

    T* pointee;        /* wrapped pointer */

    int references;    /* number of references */

  }* proxy;

protected:

  void set(T* pointee) {
    decrement();
    
    proxy = new Proxy(pointee);
  }

};
 
template<class T>
inline ref_ptr<T>::ref_ptr(T* pointee) : proxy(new Proxy(pointee)) {
}

template<class T>
inline ref_ptr<T>::ref_ptr(const ref_ptr& rhs) : proxy(rhs.proxy) {
  increment();
}

template<class T>
inline ref_ptr<T>::~ref_ptr() {
  decrement();
}

template<class T>
inline T *ref_ptr<T>::operator->() const {
  T* pointee = proxy->getPointee();;
  
  if (!pointee) {
    throw Error("ref_ptr:operator->:unitialized");
  } 
  
  return pointee;
}

template<class T>
inline ref_ptr<T>::operator void*() const {
  return proxy->getPointee();;
}

template<class T>
inline T *ref_ptr<T>::get() const {
  T* pointee = proxy->getPointee();;

  if (!pointee) {
    throw Error("ref_ptr:get:unitialized");
  } 

  return pointee;
}

template<class T> 
inline ref_ptr<T>& ref_ptr<T>::operator=(const ref_ptr& rhs) {
  if (this != &rhs) {
    decrement();
    proxy = rhs.proxy;
    increment();
  }
  
  return *this;
}

} // end of namespace Akenti

#endif
