#include <sstream>
#include <iostream>
#include <string>
#include <unistd.h>
#include "Provider.hpp"
#include "OpenSSLProviderImpl.hpp"
#include "DeletePolicyCertificateRequest.hpp"
#include "DeletePolicyCertificateResponse.hpp"
#include "ProtocolHandler.hpp"
#include "Properties.hpp"
#include "StringUtil.hpp"
#include "SSLContext.hpp"
#include "XMLUtil.hpp"

using namespace Akenti;
using namespace std;

/**
  * <pre>
  * <b>RemoteDeletePolicyCertificate</b>
  *   Use this program to remotely delete a policy certificate.
  * </pre>
  */
namespace RemoteDeletePolicyCertificate {

void printPurpose() {
  cerr << "Use this program to remotely delete a policy certificate." << endl 
       << endl;
}

void printUsage () {
  cerr << "Usage: RemoteDeletePolicyCertificate args" << endl
       << "  -F arg          - akenti client config file" << endl
       << "  [-r arg]        - resource" << endl
       << "  [-v]            - verbose" << endl
       << "  [-h]            - prints this message" << endl;
}

int 
main(int argc, char** argv) {
  if (argc < 2) {
    printPurpose();
    printUsage();
    exit(2);
  }

  string confFileName;
  string resource;
  bool verbose = false;
  int opt;

  while ((opt = getopt(argc,argv,"F:r:vh")) != EOF) {
    switch(opt){
      case 'F':
        confFileName = optarg;
        break;
      case 'r':
        resource = optarg;
        break;
      case 'v':
        verbose = true;
        break;
      case 'h':
      default:
        printUsage();
        exit(2);
    }
  }

  try {
    if (verbose) {
      cout  << "Initiizing XML library ... " << endl;
    }

    XMLUtil::init();

    if (verbose) {
      cout  << "Installing security provider ..." << endl;
    }
    
    Provider po = Provider(new OpenSSLProviderImpl);
 
    po->loadSignatureAlgorithms();
    po->seedRandomGenerator("WE GOT TO SEED THE RANDOM GENERATOR");
    Provider::setProvider(po);

    if (verbose) {
      cout  << "Loading client configuration file ..." << endl;
    }

    Properties& props = Properties::getSystemProperties();
 
    props.load(confFileName);

    if (StringUtil::equalsIgnoreCase(props.getValue("SSLMode"), "on")) {
      if (verbose) {
        cout  << "Setting SSL context file ..." << endl;
      }

      SSLContext ctx = po->createSSLContext(SSLConstants::TLS1, true);
 
      ctx->setVerify(SSLConstants::VERIFY_PEER);
      ctx->setPrivateKeyFile(props.getValue("ClientKeyFile"), SSLConstants::PEM);
      ctx->setCertificateFile(props.getValue("ClientCertFile"), SSLConstants::PEM);
 
      string path = props.getValue("CertificateAuthorityPath");
 
      if (path.length() != 0) {
        path += "/";
      }
 
      path += props.getValue("CertificateAuthorityFile");
      ctx->setTrustedCAS(path);
      SSLContext::setDefaultSSLClientContext(ctx);
    }

    if (!resource.length()) {
      cerr << "Must provide a resource" << endl;
      exit(2);
    }

    DeletePolicyCertificateRequest req;
    DeletePolicyCertificateResponse resp;

    req.setID(1);
    req.setResource(resource);

    ProtocolHandler::getInstance(ProtocolHandler::CLIENT).handle(req, resp);

    AkentiMessage mesg = resp.getAkentiMessage();

    cout << mesg.toString() << endl;

    exit(mesg.getCode() >= 200);
  } catch(Exception& ex) {
    cerr << "Exception:" << ex << endl;
    exit(2);
  } catch(Error& err) {
    cerr << "ERROR:Report to akenti team:" << err << endl;
    exit(3);
  } 
}

}
