#ifndef AK_CERTIFICATE_REQUEST_IMPL_HPP
#define AK_CERTIFICATE_REQUEST_IMPL_HPP

#include <string>
using std::string;

namespace Akenti {

class PublicKey;
class PrivateKey;
class DistinguishedName;

/**
  * This abstract class provides the methods for getting the information about 
  * a CertificateRequest such as subject, publicKey, versionNumber,
  * signatureAlgorithm or signature
  *
  * @see CertificateRequest
  * @see Signature
  *
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

class CertificateRequestImpl {

public:

  /**
    * Deletes the object
    */
  virtual ~CertificateRequestImpl() {
  }

  /**
    * Returns a <i>DistinguishedName</i> object which includes the
    * <i>DistinguishName</i>
    */
  virtual DistinguishedName getSubject() const = 0;

  /** 
    * Returns a <i>PublicKey</i> object which contains the public 
    * key of the subject of this certificateRequest.
    *
    * @see KeyPair
    * @see KeyPair#getPublicKey()
    * @see KeyFactory
    * @see KeyFactoryImpl#getKeyEncoded(const string&, const PublicKey&)
    * @see KeyFactoryImpl#initPublicKey(const string&, 
    *                                   const string&, const string&)
    * @see KeyFactoryImpl#generateRSAKeyPair(int, unsigned long)
    * @see KeyFactoryImpl#generateDSAKeyPair(int, const string&)
    * @see Signature
    * @see SignatureImpl#initVerify(const PublicKey&)
    */
  virtual PublicKey getPublicKey() const = 0;

  /**
    * Returns the versionNumber of the software that generated this 
    * <i>CertificateRequest</i>.
    */
  virtual long getVersionNumber() const = 0;

  /**
    * Returns the signatureAlgorithm such as "RSA-SHA1" or "DSA-MD5"
    * of this CertificateRequest
    */
  virtual string getSignatureAlgorithm() const = 0;

  /**
    * Returns the part of the request that was signed as a string.
    */
  virtual string getSignable() const = 0;

  /**
    * Returns the signature of this CertificateRequest
    */
  virtual string getSignature() const = 0;

  /**
    * Displays the CertificateRequest in string
    */
  virtual string paramString() const = 0;

  /**
    * Verifies that the certificate request was signed with the public key that
    * it contains.
    * 
    * @return an indication whether or not the certificate verified
    */
  virtual bool verify() const = 0;

protected:

  /**
    * Constructs the object
    */
  CertificateRequestImpl() {
  }

};

} // end of namespace Akenti

#endif
