#ifndef AK_STREAM_TOKENIZER_HPP
#define AK_STREAM_TOKENIZER_HPP

#include <iosfwd>
#include <string> 

using std::string;
using std::istream;

/**
  * This a simple stream tokenizer. Note that the stream object 
  * used to instantiate this object must outlive this object. 
  * This is because streams cannot be copied or assigned.
  * 
  * @version 1.1 00/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */

namespace Akenti {

class StreamTokenizer {

public:

  /**
    * Constructs this object. The default deimiters are any white space 
    *  characters. By default the delimiters are not returned as tokens.
    *
    * @param source an input stream
    * @param delimiters the set of delimiters used to parse the stream 
    * @param retTokens if true the delimiters are returned as a token,
    *                  if false they are not.
    */
  StreamTokenizer(istream& source, 
                  const string& delimiters = "\t\n\r\f ", 
                  bool retTokens = false);

  /**
    * Returns <i>true</i> if the tokenizer has more tokens.
    *
    * @return true if there are more tokens, false otherwise
    */
  bool hasMoreTokens();

  /**
    * Returns the next token.
    *
    * @exception Error if there are no more tokens
    * @return the next token
    */
  string nextToken();

  /**
    * Returns the next token. The new set of delimiters 
    * replaces the old one.
    *
    * @param delimiters the new set of delimiters use to parse the stream
    * @exception Error if there are no more tokens
    * @return the next token
    * @see nextToken()
    */
  string nextToken(const string& delims) {
    this->delimiters = delims;
    return nextToken();
  }

  /**
    * Prints the data member for <i>StreamTokenizer</i> using the format
    * [nameOfDataMember="valueOfDataMember",...].
    * The source data member was left out since it is a stream.
    * 
    * @return the string format as shown above
    */
  string paramString() const;
      
private:

  istream& source;        /* stream to be parsed */

  string delimiters;      /* the set of delimiters */

  bool retTokens;         /* Set this to true if the delimters are to be returned */

};

} // end of namespace Akenti 

#endif
